/* =========================================================================
 *
 *           Nanite Systems Advanced Research Encapsulation System
 *  
 *            Copyright (c) 2022–2023 Nanite Systems Corporation
 *  
 * =========================================================================
 *
 *  Menu Library
 *
 *  This program is covered under the terms of the ARES Software Copyright
 *  License, Section 3 (ASCL-iii). It may be redistributed or used as the
 *  basis of commercial, closed-source products so long as steps are taken
 *  to ensure proper attribution as defined in the text of the license.
 *  
 *  To see the full text of the ASCL, type 'help license' on any standard
 *  ARES distribution, or visit http://nanite-systems.com/ASCL for the
 *  current version.
 *
 *  DISCLAIMER
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS
 *  IS' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 *  TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 *  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 *  DAMAGES HOWEVER CAUSED ON ANY THEORY OF LIABILITY ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGE.
 *
 * =========================================================================
 *
 */

#include <ARES/a>
#define CLIENT_VERSION ARES_VERSION
#define CLIENT_VERSION_TAGS ARES_VERSION_TAGS

main(integer src, integer n, string m, key outs, key ins, key user) {
	if(n == SIGNAL_INVOKE) {
		list argv = split(m, " ");
		integer argc = count(argv);
		string msg = "";
		string action = gets(argv, 1);
		if(argc == 1 || action == "help") {
			msg = "Syntax: " + PROGRAM_NAME + " <action>";
		} else if(action == "build") {
			// old: menu build <category>; topic inferred from context of session
			// new: libmenu build <category> $topic
			
			// string s_fs = llLinksetDataRead("fs");
			string category = gets(argv, 2);
			string topic = gets(argv, 3); // getjs(sessions, [(string)outs, SESSION_TOPIC]);
			string menu_name;
			string buttons;
			
			if(category == "software-installed") {
				list pkgs = js2list(getdbl("pkg", ["version"])); // object
				integer pi = count(pkgs);
				while(pi > 0) {
					pi -= 2;
					string name = gets(pkgs, pi);
					string version = gets(pkgs, pi + 1);
					pkgs = alter(pkgs, [
						jsarray([name + "-" + version, 1, "m:software-installed-detail", name + "-" + version])
					], pi, pi + 1);
				}
				setdbl("m:software-installed", ["d"], jsarray(pkgs));
			} else if(category == "software-cache") {
				list pkgs = js2list(llLinksetDataRead("fs:package")); // array
				integer pi = count(pkgs);
				while(pi--) {
					string fn = gets(pkgs, pi);
					string nameversion = concat(delitem(splitnulls(fn, "."), LAST), ".");
					pkgs = [jsarray([nameversion, 1, "m:software-cached-detail", nameversion])] + pkgs;
				}
				setdbl("m:software-cached", ["d"], jsarray(pkgs));
			} else if(category == "fs") {
				list sources = jskeys(getdbl("fs", ["source"]));
				list views = jskeys(getdbl("fs", ["view"]));
				integer pi;
				if(~(pi = index(views, "parc")))
					views = delitem(views, pi);
					
				integer vmax = count(views);
				integer vi;
				while(vi < vmax) {
					string view = gets(views, vi);
					buttons = setjs(buttons, [JSON_APPEND], jsarray([
						view + " files...",
						1,
						"m:fs-view",
						view
					]));
					++vi;
				}
				
				vmax = count(sources);
				vi = 0;
				while(vi < vmax) {
					string source = gets(sources, vi);
					buttons = setjs(buttons, [JSON_APPEND], jsarray([
						source + ":>",
						1,
						"m:fs-source",
						source
					]));
					++vi;
				}
				
				menu_name = "m:fs";
			} else { // including 'source' and 'view'
				if(category == "view" || category == "source") {
					menu_name = "m:fs-" + category;
					category = topic;
				} else {
					menu_name = getdbl("fs", ["view", category, "menu"]);
				}
				
				if(menu_name != JSON_INVALID) {
					setdbl(menu_name, ["d"], "[]");
					
					list files = js2list(llLinksetDataRead("fs:" + category));
					integer fmax = count(files);
					integer fi = 0;
					while(fi < fmax) {
						string fn = gets(files, fi);
						buttons = setjs(buttons, [JSON_APPEND], jsarray([
							fn,
							4
						]));
						++fi;
					}
				} else {
					echo("[_menu] "+"no .menu definition for filesystem view '" + category + "'");
					return;
				}
			}
			
			// s_fs = "";
			setdbl(menu_name, ["d"], buttons);
			
		} else if(action == "prompt") {
			string cmd = concat(delrange(argv, 0, 1), " ");
			e_call(C_BASEBAND, E_SIGNAL_CALL,
				(string)outs + " " + (string)user + " baseband prompt " + jsobject([
					"message", "Specify parameters for the following command:\n\n\t" + cmd
					+ "\n\nhelp: http://support.nanite-systems.com/?id=2832#" + gets(argv, 2),
					"ins", ins,
					"command", cmd
				])
			);
		} else {
			msg = "Unknown action: " + action + ". See '" + gets(argv, 0) + " help' for more information.";
		}
		
		if(msg != "")
			print(outs, user, msg);
	} else if(n == SIGNAL_INIT) {
		#ifdef DEBUG
			echo("[" + PROGRAM_NAME + "] init event");
		#endif
	} else {
		echo("[" + PROGRAM_NAME + "] unimplemented signal " + (string)n + ": " + m);
	}
}

#include <ARES/program>
